library(dplyr)
library(tidyr)
library(stringr)
library(ggplot2)
library(reshape2)
# user defined variables
read_number<-1000
microsatellite_min = 20
microsatellite_max = 40
# read in data for samples to be compared

# table 1
data.raw.plasmid<-read.table("data/SLX10936_plasmid_mix.csv",header=T,sep=",",stringsAsFactors = F,row=1)

#table 1 re-format
data.edit = data.frame(t(data.raw.plasmid)) 
sample.names = row.names(data.edit)
data.edit = cbind(data.edit,sample.names)

data.final = data.edit %>%
  separate(sample.names, into = c("loci", "num", "direction", "Dinuc", "Crypt"), sep = "_") %>%
  mutate(Loci_full = paste(loci, num, sep = '_')) %>%
  separate(sample_name, into = c("sample","id1","id2","id3","locus","wt_length","mutant_length","wt","rep"),sep="_") %>%
  mutate(mutant_length = as.character(mutant_length)) %>%
  gather(Tract_Length, NumReads, 1:51) %>%
  mutate(NumReads, NumReads = as.numeric(NumReads)) %>%
  mutate(Tract_Length = str_replace(Tract_Length, "X", "")) %>%
  mutate(Tract_Length = as.numeric(Tract_Length)) %>%
  mutate(dist_reads = as.numeric(as.character(dist_reads))) %>%
  mutate(sample_number = as.numeric(as.character(sample_number))) %>%
  filter(Tract_Length >= microsatellite_min, Tract_Length <= microsatellite_max) %>%
  within(dist_reads<-ave(NumReads, sample_number, direction, Loci_full, FUN=sum)) %>%
  select(total_reads,sample_number,multiplex_group,dist_reads,direction,Crypt,Loci_full,Tract_Length,NumReads,pct_WT,slxID,mutant_length)

### calculare proportional read values
data.final = transform(data.final, propReads = NumReads / dist_reads)

is.nan.data.frame <- function(x)
  do.call(cbind, lapply(x, is.nan))

data.final[is.nan(data.final)] <- 0

## select WT reference data for use in simulations
data.ref = filter(data.final, pct_WT == 100 & Loci_full == "s9_8328" & dist_reads > 1000)
data.ref = within(data.ref, median<-ave(propReads, Tract_Length, direction, FUN=median))

### create data frame containing reference data
data.select = data.ref

ref.vec = c()

for (i in 1:length(unique(data.select$Tract_Length))) {
  
  length = unique(data.select$Tract_Length)[i]
  test.df = filter(data.select, Tract_Length == length)
  ref.vec = c(ref.vec,test.df$median[1])
  
}

## create simulated distribution using least square function
#### 1. create +1 shift vector ####
  ref = ref.vec

  # add buffers to crypt and ref frames
  ref.vec<-c(rep(0,10),ref,rep(0,10))
  
  #iterate through each possible shift
  #create column containing shifted reference and add to new.df
  shift = 1
    
  new.vector<-c(rep(0,length(ref.vec)))
  a<-1
    
  while (a <= length(ref.vec)) {
      
    b<-a+shift
      
    if (b > length(ref.vec)) {
        
      b<-length(ref.vec)
        
    }
      
    new.vector[b]<-ref.vec[a]
      
    a<-a+1
      
  }
    
  shift.vec<-new.vector
  
  
  ##### 2. create df with mixture values #####
  sim.df=data.frame(0:40)
  name.vector=c()
  
for (alpha in c(0,0.1,0.25,0.5,0.75,1)) {
      
    name<-paste("alpha",alpha,sep="_")
    name.vector<-c(name.vector,name)
    mix.temp<-(ref.vec*alpha)+(shift.vec*(1-alpha))
    sim.df=cbind(sim.df,mix.temp)
      
}
## create data frame containing simulated distribution
sim.df = sim.df[,-1]
sim.df = sim.df[11:31,]
colnames(sim.df) = name.vector

tract=data.frame(20:40)
colnames(tract) = "Tract_Length"
  
sim.df = cbind(tract,sim.df)

## prepare simulated data for plotting
sim.plot = melt(sim.df, id.vars="Tract_Length") 
colnames(sim.plot) = c("Tract_Length", "pct_WT", "median")

sim.plot = sim.plot %>%
            mutate(pct_WT = str_replace(pct_WT, "alpha_", "")) %>%
            mutate(pct_WT = as.numeric(pct_WT)*100)

new.col = data.frame(rep("simulated", nrow(sim.plot)))
colnames(new.col) = "type"

sim.plot = cbind(sim.plot,new.col)

### prepare plasmid data for plotting
ref.plot = data.final %>%
            filter(Loci_full == "s9_8328" & dist_reads > 1000 & mutant_length == "31") %>%
            select(Tract_Length, pct_WT, propReads) %>%
            within(median<-ave(propReads, Tract_Length, pct_WT, FUN=median)) %>%
            select(Tract_Length, pct_WT, median)

new.col3 = data.frame(rep("plasmid", nrow(ref.plot)))
colnames(new.col3) = "type"

ref.plot = cbind(ref.plot,new.col3)

plot.df = rbind(sim.plot,ref.plot) %>%
            mutate(pct_WT = str_replace(pct_WT, "  ", "")) %>%
            mutate(pct_WT = str_replace(pct_WT, " ", "")) %>%
            mutate(pct_WT = paste(pct_WT,"% WT",sep=""))

plot.df = plot.df %>% filter(pct_WT == '0% WT' | pct_WT == '50% WT' | pct_WT == '100% WT')

### plot Figure 2D
pp = ggplot(plot.df, aes(x=Tract_Length,y=median,color=type)) + 
  geom_point(alpha=1, size=5) + geom_line(alpha=1, size=2) +
  facet_grid(pct_WT ~ .) +
  xlab("Repeat length") + ylab("Proportion of reads") +
  theme(legend.text=element_text(size=24),legend.title=element_text(size=28),
        axis.title.x = element_text(size=28), axis.title.y = element_text(size=28),
        strip.text.x = element_text(size=24), strip.text.y = element_text(size=24),
        axis.text.x = element_text(size=24), axis.text.y = element_text(size=24)) +
  scale_color_discrete(name = "Data",labels = c("Predicted","Actual"))
plot(pp)

