library(plyr)
library(reshape2)
library(dplyr)
library(tidyr)
library(stringr)
library(ggplot2)

read_number = 1000
microsatellite_min = 20
microsatellite_max = 40

### read in raw data 
data.raw = read.table("data/msh2_data_final.csv",sep=",", header=T, stringsAsFactors=F) %>%
            #exclude transgenic locus
            filter(Loci_full != "s1_eyfp")
data.phi = read.table("data/msh2_phi_data.csv", sep=",", header=T,stringsAsFactors=F) %>%
              mutate(specific_ID = paste(slxID,mouse_ID,sep="-")) %>%
              #exclude transgenic locus
              filter(loci != "s1_eyfp")

## info on loci to be excluded
loci.exclude = read.table("loci_to_exclude_msh2.csv", sep=",", header=T, stringsAsFactors=F)

data.master = data.raw %>%
              #remove negative controls and poor performing sequencing run
              filter(mouse_ID != "neg_control" | slxID == "SLX9311") %>%
              mutate(specific_ID = paste(slxID,mouse_ID,sep="-"))

# exclude loci that have bimodal distributions
for (i in 1:nrow(loci.exclude)) {
  
  data.master = data.master %>%
    filter(slxID != loci.exclude$slxID[i] | mouse_ID != loci.exclude$mouse_ID[i] |
             Loci_full != loci.exclude$loci[i])
  
}

### identify loci represented in data
loci_all = unique(data.master$Loci_full)

#filter out reference samples
data.ref = data.master %>%
            filter(pct_WT == 100 & dist_reads > 1000)

#calculate median values at each tract length for each reference set
data.ref = within(data.ref, median<-ave(propReads, Tract_Length, specific_ID, Loci_full, FUN=median))

#calculate least squares difference at each tract length
data.ref = data.ref %>%
            mutate(ls_ind = (propReads - median)^2)

#calculate sum of least squares for each reference set (across all tract lengths)
data.ref = within(data.ref, ls_total<-ave(ls_ind, direction, specific_ID, Loci_full, FUN=sum))
data.ref = within(data.ref, ls_mean<-ave(ls_ind, direction, specific_ID, Loci_full, FUN=mean))

variability_threshold=0.0035

#generate table detailing loci and runs/mice with ref distributions with high var
bad.ref = data.ref %>%
            filter(ls_total > variability_threshold) %>%
            select(Loci_full, specific_ID) %>%
            unique()

#remove samples with reference variability higher than threshold
filter.df = data.phi

for (i in 1:nrow(bad.ref)) {
  
  reference = bad.ref[i,]
  filter.df = filter.df %>%
                filter(specific_ID != reference$specific_ID | 
                         loci != reference$Loci_full)
  
  
}

### call PPCs and WPCs
ppc.vector = c()
wpc.vector = c()

for (i in 1:nrow(filter.df)) {
  
  phi.test = filter.df$mean_phi[i]
  loci.test = filter.df$loci[i]
  sex.test = filter.df$sex[i]
  
  # determine threshold for ppc and wpc for this data sample
    ppc.bound =  0.182
      
      # give different threshold for X-linked loci and eyfp locus
      if (loci.test == "sX_1395" & sex.test == "male" | loci.test == "s1_eyfp") {
        
        wpc.bound = 0.596
        
      } else {
        
        wpc.bound = 0.299
        
      }
  
  if (phi.test <= ppc.bound) {
    
    wpc = 0
    ppc = 0
    
  } else if (phi.test < wpc.bound) {
    
    wpc = 0
    ppc = 1
    
  } else {
    
    wpc = 1
    ppc = 0
  }
  
  ppc.vector = c(ppc.vector,ppc)
  wpc.vector = c(wpc.vector,wpc)
  
}

## combined ppc and wpc calls with data.edit
ppc.df = data.frame(ppc.vector)
colnames(ppc.df) = "ppc"

wpc.df = data.frame(wpc.vector)
colnames(wpc.df) = "wpc"

data.cp = cbind(filter.df,ppc.df,wpc.df) 

### generate crypt summary data
crypt_all = unique(data.cp$crypt_ID)

ppc.total = c()
wpc.total = c()
crypt.vector = c()
mouse.vector = c()
induc.vector = c()
loci.vector = c()

for (crypt.test in crypt_all) {
  
  crypt.df = data.cp %>%
    filter (crypt_ID == crypt.test)
  
  ppc.sum = sum(crypt.df$ppc)
  wpc.sum = sum(crypt.df$wpc)
  
  crypt.vector = c(crypt.vector,crypt.test)
  mouse.vector = c(mouse.vector,crypt.df$mouse_ID[1])
  induc.vector = c(induc.vector,crypt.df$post_induction[1])
  ppc.total = c(ppc.total,ppc.sum)
  wpc.total = c(wpc.total,wpc.sum)
  loci.vector = c(loci.vector,nrow(crypt.df))
  
}

crypt.df = data.frame(crypt.vector)
colnames(crypt.df) = "crypt_ID"
mouse.df = data.frame(mouse.vector)
colnames(mouse.df) = "mouse_ID"
induc.df = data.frame(induc.vector)
colnames(induc.df) = "post_induction"
ppc.df = data.frame(ppc.total)
colnames(ppc.df) = "total_ppc"
wpc.df = data.frame(wpc.total)
colnames(wpc.df) = "total_wpc"
loci.df = data.frame(loci.vector)
colnames(loci.df) = "N_loci"

crypt.summary = cbind(crypt.df,mouse.df,induc.df,ppc.df,wpc.df,loci.df)

crypt.keep = crypt.summary 

crypt.keep = crypt.keep$crypt_ID
crypt.keep = as.character(crypt.keep)

## filter data based on crypt.keep
data.filter = data.cp %>%
  subset(crypt_ID %in% crypt.keep)

### calculate total crypts tested within each mouse
### calculate total ppc and wpc within each mouse
mouse_all = unique(data.filter$mouse_ID)
loci_all = unique(data.filter$loci)

data.final = data.frame()

for (mouse.test in mouse_all) {
  
  mouse.df = data.filter %>%
    filter (mouse_ID == mouse.test)
  
  for (loci.test in loci_all) {
    
    ppc.total = c()
    wpc.total = c()
    crypt.vector = c()
    
    loci.df = mouse.df %>%
      filter (loci == loci.test)
    
    crypt.test = nrow(loci.df)
    ppc.sum = sum(loci.df$ppc)
    wpc.sum = sum(loci.df$wpc)
    
    crypt.vector = c(crypt.vector,rep(crypt.test,crypt.test))
    ppc.total = c(ppc.total,rep(ppc.sum,crypt.test))
    wpc.total = c(wpc.total,rep(wpc.sum,crypt.test))
    
    crypt.df = data.frame(crypt.vector)
    colnames(crypt.df) = "crypts_tested"
    ppc.df = data.frame(ppc.total)
    colnames(ppc.df) = "total_ppc"
    wpc.df = data.frame(wpc.total)
    colnames(wpc.df) = "total_wpc"
    
    final.df = cbind(loci.df,crypt.df,ppc.df,wpc.df)
    
    data.final = rbind(data.final,final.df)
    
  }
  
}

# calculate pct wpc and pct ppc in each sequencing run
data.final = transform(data.final, pct_ppc = total_ppc / crypts_tested *100)
data.final = transform(data.final, pct_wpc = total_wpc / crypts_tested *100)

# create summary data
data.summary = data.final %>%
  select(post_induction, mouse_ID,loci,crypts_tested,pct_ppc,pct_wpc) %>%
  mutate(post_induction, post_induction = as.numeric(as.character(post_induction))) %>%
  # remove loci with < 3 time points
  filter(loci != "a10_2051", loci != "s8_6742") %>%
  unique() %>%
  # remove samples with <60 crypts analysed
  filter(crypts_tested > 60) %>%
  # filter start time point + end time point
  filter(post_induction != 0 & post_induction != 164)

summary.table = data.summary %>% select(mouse_ID,post_induction,loci,crypts_tested,pct_ppc,pct_wpc)

#SUPPLEMENTARY TABLE 6
write.table(summary.table, file="msh2_locus_summary.csv", sep=",",row.names=F)

data.summary = ddply(data.summary, c("mouse_ID","post_induction"), summarise,
                     N_pct_ppc    = length(pct_ppc),
                     mean_pct_ppc = mean(pct_ppc),
                     sd_pct_ppc   = sd(pct_ppc),
                     se_pct_ppc   = sd_pct_ppc / sqrt(N_pct_ppc),
                     
                     N_pct_wpc    = length(pct_wpc),
                     mean_pct_wpc = mean(pct_wpc),
                     sd_pct_wpc   = sd(pct_wpc),
                     se_pct_wpc   = sd_pct_wpc / sqrt(N_pct_wpc)                                 
                     
)

# plot figure 3F
plot.df = ddply(data.cp, c("post_induction","loci"), summarise,
                 
                 total_wpc = sum(wpc),
                 total_ppc = sum(ppc),
                 total_crypts = length(wpc),
                 total_mutants = sum(total_wpc,total_ppc),
                 
                 pct_wpc = total_wpc / total_crypts * 100,
                 pct_ppc = total_ppc / total_crypts * 100,
                 pct_mutants = total_mutants / total_crypts * 100
) %>%
  filter(post_induction != 0 & post_induction != 164)

loci.summary = ddply(plot.df, c("loci"),summarise,
                     
                     N_mice = length(post_induction)) %>%
                        filter(N_mice >= 6)

loci.final = loci.summary$loci

plot.df = filter(plot.df, loci %in% loci.final)

pp = plot.df %>%
  ggplot(aes(x=loci, y = pct_mutants)) +
  geom_boxplot(size=1) +
  xlab("Locus") + ylab("Mutant crypts (%)") +
  theme(legend.text=element_text(size=24),legend.title=element_text(size=28),
        axis.title.x = element_text(size=28), axis.title.y = element_text(size=28),
        strip.text.x = element_text(size=24), strip.text.y = element_text(size=24),
        axis.text.x = element_text(size=14), axis.text.y = element_text(size=24))

plot(pp)

# clean data for plotting
plot.df2 = data.summary 
plot.df2[sapply(plot.df2,is.na)] = 0

## plot figure 3F
pp2 = ggplot(data=plot.df2, aes(x=post_induction, y=mean_pct_ppc,colour="PPC")) +
  geom_point(size=5) +
  geom_errorbar(aes(ymin = mean_pct_ppc - se_pct_ppc, ymax = mean_pct_ppc + se_pct_ppc, colour="PPC"), width=2) +
  geom_smooth(aes(post_induction,mean_pct_ppc, colour="PPC"), method=lm, se=FALSE, size=2) +
  geom_point(aes(x=post_induction,y=mean_pct_wpc,colour="WPC"),size=5) +
  geom_errorbar(aes(ymin=mean_pct_wpc-se_pct_wpc, ymax=mean_pct_wpc+se_pct_wpc, colour="WPC"), width=2) +
  geom_smooth(aes(post_induction,mean_pct_wpc, colour="WPC"), method=lm, se=FALSE, size=2) +
  ylab("Percentage of crypts tested (%)") + xlab("Time after Msh2 loss (days)") +
  scale_colour_manual(name = 'Clone\naccumulation', values =c('WPC' = '#0072B2','PPC' = '#D55E00')) +
  theme(legend.text=element_text(size=24),legend.title=element_text(size=28),
        axis.title.x = element_text(size=28), axis.title.y = element_text(size=28),
        strip.text.x = element_text(size=24), strip.text.y = element_text(size=24),
        axis.text.x = element_text(size=24), axis.text.y = element_text(size=24))

plot(pp2)
